clear all;
format long;

%---------------------------------------------------------------------------------------
% This code plots Pd vs. Pf for a given SNR by varying the detection threshold
%---------------------------------------------------------------------------------------
%                               non-coherent mode
%   a. N    = 31        %effective code length (number of ON pulses in the ternary code)
%   c. L    = 8         %noncoherent average over an integer number of codes
%   d. GbdB = 13        %estimated EbN0 (dB) before square law detector
%---------------------------------------------------------------------------------------

%parameters
%==========
N       = 16;                           %effective code length (number of ON pulses in the ternary code)
L       = 8;                            %noncoherent average of L codes
alpha   = 1;                            %fraction of multipath energy captured in main path
GbdB    = 8;                            %EbN0 per symbol or code (dB) (before square-law, i.e. coherent)
margin  = 0;                            %margin to account for implementation losses
GpdB    = GbdB - 10*log10(N) - margin;  %EpN0 per pulse (dB) (average)
Gp      = 10^(GpdB/10);                 %EpN0 per pulse (Linear)

nn      = round(log2(N));
Tmax    = 6*L*N*Gp;                     %maximum threshold value
dT      = Tmax/256;                     %threshold step
Tv      = 64*dT:dT:Tmax;LT = length(Tv); %vary threshold from zero to maximum value
PD      = zeros(1,LT);
PF      = zeros(1,LT);
i = 0;Flag = 0;
while ((i < LT) & (Flag == 0));
    i = i + 1;T = Tv(i);
    PD(i) = wbaMarcumq(sqrt(2*L*N*Gp*alpha),sqrt(2*T),L*N);
    LogT = log(T);LogX = -T;Z = exp(-T);
    for k = 1:L*N-1, LogX = LogX + LogT - log(k);Z = Z + exp(LogX);end;PF(i) = Z;
    for m = 1:nn, PF(i) = 2*PF(i) - PF(i)*PF(i);end;
    if (PF(i) < 1e-3), Flag = 1;end;
    fprintf('i = %d, Threshold = %f, Pd = %f, Pf = %f\n',i,T,PD(i),PF(i));
end;

figure;plot(PF,PD);
grid;xlabel('False alarm probability P_F');ylabel('Detection probability P_D');
title('PD vs. PF');axis([0 0.25 0.75 1]);